INCLUDE "hardware.inc"
INCLUDE "input.inc"

Section "Player Data", WRAM0

playerWorldX: dw
playerWorldY: dw

Section "Player Code", ROM0

spriteData:
    INCBIN "png/player.2bpp"

DEF SPRITE_IDX EQU 32
DEF SPRITE_WIDTH EQU 2
DEF SPRITE_HEIGHT EQU 2

Player_Init::
    ld a, 8
    ld hl, _OAM + 1
    ld [hl], a

    ld bc, _VRAM8000 + SPRITE_IDX * 16
    ld hl, spriteData
    ld d, 16 * (SPRITE_WIDTH * SPRITE_HEIGHT)
    call memcpy

    ; top left
    ld hl, _OAM
    ld a, 144-8-8
    ld [hli], a
    ld a, 8
    ld [hli], a
    ld a, SPRITE_IDX
    ld [hli], a
    xor a
    ld [hli], a

    ; top right
    ld a, 144-8-8
    ld [hli], a
    ld a, 8 + 8
    ld [hli], a
    ld a, SPRITE_IDX + 1
    ld [hli], a
    xor a
    ld [hli], a

    ; bottom left
    ld a, 144-8
    ld [hli], a
    ld a, 8
    ld [hli], a
    ld a, SPRITE_IDX + 2
    ld [hli], a
    xor a
    ld [hli], a

    ; bottom right
    ld a, 144-8
    ld [hli], a
    ld a, 8 + 8
    ld [hli], a
    ld a, SPRITE_IDX + 3
    ld [hli], a
    xor a
    ld [hli], a

    ld bc, _OAMRAM
    ld hl, _OAM
    ld d, 4 * (SPRITE_WIDTH * SPRITE_HEIGHT)
    call memcpy

    ret

Player_Update::
    ; Only update every 2 frames
    ld hl, frame
    ld a, [hl]
    and %1
    cp %1
    jr nz, .done

    ld hl, keys
    ld b, [hl]

    ; check for move right
    ld a, b
    and BTN_RIGHT
    jr z, .left

    ; check for right boundary
    ld hl, _OAM + 1
    ld a, [hl]
    cp 160 - 8
    jr nc, .done

    ; update top left sprite
    inc [hl]
    inc hl
    ld [hl], SPRITE_IDX
    inc hl
    res 5, [hl]

    ; update top right sprite
    inc hl
    inc hl
    inc [hl]
    inc hl
    ld [hl], SPRITE_IDX + 1
    inc hl
    res 5, [hl]

    ; update bottom left sprite
    inc hl
    inc hl
    inc [hl]
    inc hl
    ld [hl], SPRITE_IDX + 2
    inc hl
    res 5, [hl]

    ; update bottom right sprite
    inc hl
    inc hl
    inc [hl]
    inc hl
    ld [hl], SPRITE_IDX + 3
    inc hl
    res 5, [hl]

.left:
    ; check for left button
    ld a, b
    and BTN_LEFT
    jr z, .done

    ; check for left boundary
    ld hl, _OAM + 1
    ld a, [hl]
    cp 9
    jr c, .done

    ; update top left sprite
    dec [hl]
    inc hl
    ld [hl], SPRITE_IDX + 1
    inc hl
    set 5, [hl]

    ; update top right sprite
    inc hl
    inc hl
    dec [hl]
    inc hl
    ld [hl], SPRITE_IDX
    inc hl
    set 5, [hl]

    ; update bottom left sprite
    inc hl
    inc hl
    dec [hl]
    inc hl
    ld [hl], SPRITE_IDX + 3
    inc hl
    set 5, [hl]

    ; update bottom right sprite
    inc hl
    inc hl
    dec [hl]
    inc hl
    ld [hl], SPRITE_IDX + 2
    inc hl
    set 5, [hl]

.done:
    ret
